/**
 * $Id: JPort.java,v 1.2 2005/10/23 20:10:46 mviara Exp $
 * $Name:  $
 *
 * $Log: JPort.java,v $
 * Revision 1.2  2005/10/23 20:10:46  mviara
 * Added support for persistent data and input on port.
 *
 * Revision 1.1.1.1  2004/08/22 06:46:25  mviara
 * 8051 emulator
 *
 */
package j51.intel;

import java.awt.*;
import javax.swing.*;
import java.util.Vector;

import j51.util.Hex;
import j51.swing.*;

/**
 *
 * 8051 standard I/O port implementation.
 *
 */
public class JPort extends JPanel implements MCS51Peripheral,SfrWriteListener,MCS51Constants
{
	private int sfrPort[];
	private int sfrM1[];
	private int sfrM2[];
	private int disabled[];
	
	private BitListener box[][];
	private JHexByte pm[][];
	private int numPort;
	
	class BitListener extends JCheckBox  implements javax.swing.event.ChangeListener, MCS51Peripheral
	{
		private int r,b,mask;
		MCS51 cpu;
		
		BitListener(int r,int b)
		{
			this.r = r;
			this.b = b;
			cpu = null;
			mask = 1 << b;
			addChangeListener(this);
		}

		public void setEnabled(boolean enabled)
		{

			removeChangeListener(this);
			if (enabled)
				addChangeListener(this);
			super.setEnabled(enabled);
		}
		
		public void registerCpu(MCS51 cpu)
		{
			this.cpu = cpu;
		}
		
		public void stateChanged(javax.swing.event.ChangeEvent e)
		{
			class SfrChange implements Runnable
			{
				boolean mode;
				
				SfrChange(boolean mode)
				{
					this.mode = mode;
				}

				public void run()
				{
					int sfr = cpu.sfr(0x80+16*r);
					if (mode )
						sfr |= mask;
					else
						sfr &= ~mask;
					cpu.sfr(0x80+16*r,sfr);
				}

				
			}

			int newMask = mask;
			if   (!isSelected() )
				newMask = 0;

			if ((cpu.sfr(0x80+16*r) & mask) != newMask)
			{
				if (cpu != null)
					cpu.addRunQueue(new SfrChange(isSelected()));
			}
		}
	}
	
	public JPort(int numPort)
	{
		super(new GridBagLayout());
		JFactory.setTitle(this,"Port 0-"+numPort);
		this.numPort = numPort;
		this.sfrPort = new int[numPort];
		this.sfrM1		= new int[numPort];
		this.sfrM2		= new int[numPort];
		this.disabled	= new int[numPort];
		
		this.box = new BitListener[numPort][8];
		this.pm	= new JHexByte[numPort][2];

		GridBagConstraints g = new GridBagConstraints();
		g.gridx = 0; g.gridy = 0;g.gridwidth = 1;g.gridheight = 1;
		g.anchor = g.CENTER; g.fill = g.NONE; 


		g.gridx = 1;
		for (int i = 0; i < 8 ; i++)
		{
			add(new JLabel(""+(7-i)),g);
			g.gridx++;
			
		}
		
		add(new JLabel("M1"),g);
		g.gridx++;
		add(new JLabel("M2"),g);

		g.gridy++;

		for (int port = 0 ; port < sfrPort.length ; port++)
		{
			disabled[port] = 0;
			pm[port][0] = new JHexByte();
			pm[port][1] = new JHexByte();

			switch (port)
			{
				case	0:
						sfrPort[port]	= P0;
						sfrM1[port]		= P0M1;
						sfrM2[port]		= P0M2;
						break;
				case	1:
						sfrPort[port]	= P1;
						sfrM1[port]		= P1M1;
						sfrM2[port]		= P1M2;
						break;
				case	2:
						sfrPort[port] = P2;
						sfrM1[port]		= P2M1;
						sfrM2[port]		= P2M2;
						break;
				case	3:
						sfrPort[port]	= P3;
						sfrM1[port]		= P3M1;
						sfrM2[port]		= P3M2;
						break;
			}

			g.gridx = 0;
			add(new JLabel("P"+port),g);
			g.gridx++;
			for (int i = 0 ; i < 8 ; i++)
			{
				
				box[port][7-i] = new BitListener(port,7-i);
				add(box[port][7-i],g);
				g.gridx++;
				
			}
			
			add(pm[port][0],g);
			g.gridx++;
			add(pm[port][1],g);

			g.gridy++;

			
		}


	}

	public void registerCpu(MCS51 cpu)
	{
		int i ;

		for (int port = 0 ; port < numPort ; port++)
			for (i = 0 ; i < 8 ; i++)
				box[port][i].registerCpu(cpu);

		for (i = 0 ; i < sfrPort.length ; i++)
		{
			cpu.addSfrWriteListener(sfrPort[i],this);
			cpu.addSfrWriteListener(sfrM1[i],this);
			cpu.addSfrWriteListener(sfrM2[i],this);
		}
	}

	public void sfrWrite(int reg,int value)
	{
		for (int i = 0 ; i < sfrPort.length ; i++)
		{
			if (reg == sfrPort[i])
			{
				update(i,value);
				break;
			}

			if (reg == sfrM1[i])
			{
				M1(i,value);
				break;
			}

			if (reg == sfrM2[i])
			{
				M2(i,value);
				break;
			}

		}
	}

	void mode(int port)
	{
		for (int i = 0 ; i < 8 ; i++)
		{
			int mask = 1 << i;

			boolean  m1 = false , m2 = false;
			boolean dis = (disabled[port] & mask) != 0;
			
			try
			{

				m1 = (pm[port][0].getValue() & mask) != 0;
				m2 = (pm[port][1].getValue() & mask) != 0;
			}
			catch (Exception e)
			{
			}

			if (dis)
				box[port][i].setEnabled(false);
			else
			{
				if (!m1 && m2)
					box[port][i].setEnabled(false);
				else
					box[port][i].setEnabled(true);
			}
		}
	}

	public void setDisableMask(int port,int mask)
	{
		disabled[port] = mask;
		for (int i = 0 ; i < 8 ; i ++)
			if ((mask & (1 << i)) != 0)
				box[port][i].setEnabled(false);
	}
	
	void M1(int port,int value)
	{
		pm[port][0].setValue(value);
		mode(port);
	}

	void M2(int port,int value)
	{
		pm[port][1].setValue(value);
		mode(port);
	}

	void update(int port,int value)
	{
		for (int i = 0 ; i < 8 ; i++)
		{
			boolean mode = (value & (1 << i)) != 0;
			if (box[port][i].isSelected() != mode)
				box[port][i].setSelected(mode);
		}
	}
}


