/**
 * $Id: Timer.java,v 1.1.1.1 2004/08/22 06:46:25 mviara Exp $
 * $Name:  $
 *
 * $log$
 *
 */

package j51.intel;

import j51.util.Hex;

/**
 *
 * Standard 8051 timer 0 and 1 implementation. Only mode 0,1,2 are
 * supported and only in the timer mode no counter or gate enable are
 * supported.
 *
 */
public class Timer implements MCS51Peripheral,MachineCyclesListener,MCS51Constants
{
	class SingleTimer implements InterruptSource,MCS51Constants
	{
		private MCS51 cpu;
		private int tconShift;
		private int tmodShift;
		private int TH;
		private int TL;
		private int TF;
		private int ET;
		private int vector;
		private int timer;

		SingleTimer(int timer)
		{
			tconShift = 2 * timer;
			tmodShift = 4 * timer;
			TH = MCS51Constants.TH0 + timer;
			TL = MCS51Constants.TL0 + timer;
			TF = TCON_TF0 << tconShift;
			ET = timer == 0 ? IE_ET0 : IE_ET1;
			vector = 0x0b + timer * 0x10;
			this.timer = timer;
		}


		void registerCpu(MCS51 cpu)
		{
			this.cpu = cpu;
			cpu.addInterruptSource(MCS51Constants.IE,this);
			cpu.addInterruptSource(MCS51Constants.TCON,this);

		}

		void cycle()
		{
			int tl,th;

			// Do nothing if timer not running
			int tcon = cpu.sfr(MCS51Constants.TCON) >> tconShift;
			if ((tcon & TCON_TR0) == 0)
				return;

			int tmod = cpu.sfr(TMOD) >> tmodShift;

			switch (tmod & (TMOD_T0_M0 | TMOD_T0_M1))
			{
				case	0:	// 13 bit timer
					tl = (cpu.sfr(TL) + 1) & 0x1f;
					cpu.sfr(TL,tl);

					if (tl == 0)
					{
						th = (cpu.sfr(TH) + 1) & 0xff;
						cpu.sfr(TH,th);
						if (th == 0)
						{
							cpu.sfrSet(TCON,TF);
							//System.out.println("Timer"+timer+" TF13");
						}
					}

					break;
				case	TMOD_T0_M0:
					tl = (cpu.sfr(TL) + 1) & 0xff;
					cpu.sfr(TL,tl);
					if (tl == 0)
					{
						th = (cpu.sfr(TH) + 1) & 0xff;
						cpu.sfr(TH,th);
						if (th == 0)
						{
							//System.out.println("Timer"+timer+"TF16");

							cpu.sfrSet(TCON,TF);
						}
					}
					break;
				case	TMOD_T0_M1:
					tl = (cpu.sfr(TL) + 1) & 0xff;

					if (tl == 0)
					{
						cpu.sfr(TL,cpu.sfr(TH));
						cpu.sfrSet(TCON,TF << tconShift);

					}
					else
						cpu.sfr(TL,tl);
					break;

			}

		}

		public void interruptStart()
		{
			cpu.sfrReset(TCON,TF);
		}

		public void interruptStop()
		{
		}

		public boolean interruptCondition()
		{
			if ((cpu.sfr(IE) & ET) != 0 && (cpu.sfr(TCON) & TF) != 0)
				return true;

			return false;
		}

		public int getInterruptVector()
		{
			return vector;
		}

		public String toString()
		{
			return "Timer"+timer+" at "+Hex.bin2word(vector);
		}
	}
	
	private MCS51 cpu;
	private SingleTimer timer0 = new SingleTimer(0);
	private SingleTimer timer1 = new SingleTimer(1);
	
	public void registerCpu(MCS51 cpu)
	{
		this.cpu = cpu;
		cpu.addMachineCycleListener(this);
		timer0.registerCpu(cpu);
		timer1.registerCpu(cpu);
	}
	
	public void cycles(int n)
	{
		while (n-- > 0)
		{
			timer0.cycle();
			timer1.cycle();
		}
	}
}
