/**
 * $Id: LPC764Base.java,v 1.2 2005/11/02 11:33:11 mviara Exp $
 * $Name:  $
 *
 * LPC764 Emulator emulated peripheral :
 * 
 * 4KB OTP	Persistent
 * EEPROM	Persistent
 * JUART
 * TIMER
 * 
 * $Log: LPC764Base.java,v $
 * Revision 1.2  2005/11/02 11:33:11  mviara
 * Removed some debug code.
 *
 * Revision 1.1  2005/11/02 08:20:25  mviara
 * Split LPC764 in LPC764Base.
 *
 *
 */
package j51.philips;

import j51.intel.*;

public class LPC764Base extends MCS51 implements Code
{
	private PersistentBuffer flash;
	private PersistentBuffer eeprom;

	static public final int EEPROM_START = 0xfc00;
	static public final int EEPROM_SIZE  = 0x0400;
	static public final int EEPROM_UCFG1 = 0xfd00;
	static public final int EEPROM_UCFG2 = 0xfd01;
	
	public LPC764Base(String name)
	{
		oscillator = 6000000;
		machineCycles = 6;


		flash = new PersistentBuffer(name,"flash",4096);
		eeprom= new PersistentBuffer(name,"eeprom",768);
		
		setCode(this);

		addPeripheral(new JUart());
		addPeripheral(new j51.intel.Timer());

	}

	public void setCodeSize(int size)
	{
	}

	public int getCodeSize()
	{
		return 4096;
	}

	public void setCode(int addr,int value)
	{
		if (addr >= EEPROM_START)
		{
			//System.out.println("WRITE EEPROM "+j51.util.Hex.bin2word(addr)+" = "+j51.util.Hex.bin2byte(value));
			eeprom.set(addr-EEPROM_START,value);

		}
		else
			flash.set(addr,value);
	}

	public int getCode(int addr)
	{
		int value;
		if (addr >= EEPROM_START)
		{
			value = eeprom.get(addr-EEPROM_START);
			//System.out.println("READ EEPROM "+j51.util.Hex.bin2word(addr)+" = "+j51.util.Hex.bin2byte(value));
		}
		else
			value = flash.get(addr);

		return value;
	}



	public String toString()
	{
		return "Diseqc $Id: LPC764Base.java,v 1.2 2005/11/02 11:33:11 mviara Exp $";
	}
	
}
