/**
 * $Id: LPC900.java,v 1.3 2005/10/28 15:33:25 mviara Exp $
 * $Name:  $
 *
 * LPC900 Family emulator.
 *
 * Emulated periopheral :
 *
 * - Flash memory
 * - Eeprom
 * - Double DPTR
 * 
 * $Log: LPC900.java,v $
 * Revision 1.3  2005/10/28 15:33:25  mviara
 * Added supporto for multiple DPTR.
 *
 * Revision 1.2  2005/10/23 20:11:21  mviara
 * First running version for LPC900.
 *
 */
package j51.philips;

import j51.intel.*;

class LPC900Constants
{
	static public final int FMCON	= 0xe4;
	static public final int FMDATA	= 0xe5;
	static public final int FMADRL	= 0xe6;
	static public final int FMADRH	= 0xe7;
	static public final int DEECON	= 0xf1;
	static public final int DEEDAT  = 0xf2;
	static public final int DEEADR  = 0xf3;
	static public final int AUXR1	= 0xa2;
}



class LPC900Port extends JPort
{
	LPC900Port()
	{
		super(4);
		setDisableMask(3,0xFC);
	}
}


public class LPC900 extends MCS51 implements SfrWriteListener
{
	private byte flashBuffer[];
	private int  flashSize,flashCounter;
	private int FMCON;
	private int FMADRL;
	private int FMADRH;
	private int FMDATA;
	private PersistentBuffer eeprom;
	private boolean eepromWrite = false;
	
	public LPC900()
	{
		eeprom = new PersistentBuffer("LPC900","eeprom",512);

		
		oscillator = 7372800;
		machineCycles = 2;

		addPeripheral(new LPC900Port());
		addPeripheral(new Timer());
		addPeripheral(new JUart());

		/**
		 * Flash interface
		 */
		addSfrWriteListener(LPC900Constants.FMCON,this);
		addSfrWriteListener(LPC900Constants.FMDATA,this);
		addSfrWriteListener(LPC900Constants.DEEDAT,this);
		addSfrWriteListener(LPC900Constants.DEEADR,this);
		addSfrWriteListener(LPC900Constants.AUXR1,this);

		/**
		 * Add Flash memory
		 */
		setCode(new FlashCode("LPC900",8192));

		flashBuffer = new byte[64];

		setSfrName(LPC900Constants.FMDATA,"FMDATA");
		setSfrName(LPC900Constants.FMCON,"FMCON");
		setSfrName(LPC900Constants.FMADRL,"FMADRL");
		setSfrName(LPC900Constants.FMADRH,"FMADRH");
		setSfrName(LPC900Constants.DEECON,"DEECON");
		setSfrName(LPC900Constants.DEEDAT,"DEEDAT");
		setSfrName(LPC900Constants.DEEADR,"DEEADR");

	}


	public void sfrWrite(int r,int v)
	{
		switch (r)
		{
			case	LPC900Constants.AUXR1:
				
				// Bit 2 is always 0
				v &= 0xfb;
				sfr(LPC900Constants.AUXR1,v);
				swapDptr(v & 1);
				break;
				
			case	LPC900Constants.DEEDAT:
				eepromWrite = true;
				break;
				
			case	LPC900Constants.DEEADR:
				int addr = v + (sfr(LPC900Constants.DEECON) & 1) * 256;
				if (eepromWrite)
				{
					eeprom.set(addr,sfr(LPC900Constants.DEEDAT));
				}
				else
				{
					sfr(LPC900Constants.DEEDAT,eeprom.get(addr));
				}
				sfr(LPC900Constants.DEECON,sfr(LPC900Constants.DEECON)|0x80);
				eepromWrite = false;
				break;
				
			case	LPC900Constants.FMDATA:
				flashBuffer[flashCounter++] = (byte)v;
				break;
				
			case	LPC900Constants.FMCON:
				switch (v)
				{
					case	0x00:
						flashCounter = 0;
						break;
					case	0x68:
						int base = sfr(LPC900Constants.FMADRL) +
							   sfr(LPC900Constants.FMADRH) * 256;
						
						for (int i = 0 ; i < flashCounter ; i++)
							code(base+i,flashBuffer[i]);
						break;
						
				}
		}
			
	}
	
	public String toString()
	{
		return "Philips 89LPC900 $Id: LPC900.java,v 1.3 2005/10/28 15:33:25 mviara Exp $";
	}
}
